<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use PhpSpec\ObjectBehavior;

/**
 * @method parse(\SimpleXMLElement $simplexml_load_string)
 * @method addForbiddenTags(string[] $array)
 * @method setForbiddenTags(string[] $array)
 * @method setAllowedTags(string[] $array)
 */
class ElementParserSpec extends ObjectBehavior
{
    function it_is_initializable()
    {
        $this->shouldHaveType(ElementParser::class);
    }

    function it_implements_correct_interfaces()
    {
        $this->shouldImplement(ItemParser::class);
    }

    public function it_should_parse_an_element()
    {
        $supportedText = 'Lorem ipsum dolor sit amet, consectetur adipisicing elit. Dolorem earum magni quam sapiente similique voluptatibus? A aliquam aut beatae fugit, in repellendus vero. Aliquam pariatur possimus quae quibusdam ratione unde?';
        $xml = <<<XML
<element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">$supportedText</element>
XML;
        $this->validateXML($xml, [
            'id' => 'paragraph-273ee570c1469bdf5badeea5f0524166',
            'type' => 'body',
            'content' => $supportedText
        ]);
    }

    public function it_should_should_support_single_tags()
    {
        $content = 'Texts with <br/> and <hr/>';
        $xml = <<<XML
<element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">$content</element>
XML;
        $this->validateXML($xml, [
            'id' => 'paragraph-273ee570c1469bdf5badeea5f0524166',
            'type' => 'body',
            'content' => $content
        ]);
    }

    public function it_should_allow_certain_tags()
    {
        $notAllowedTags = [
            'p',
            'div',
        ];

        $notAllowedTagsXML = implode(' ', array_map(static function ($tag) {
            return "<$tag>No tag</$tag>";
        }, $notAllowedTags));

        $notAllowedTagsParsed = implode(' ', array_map(static function ($tag) {
            return 'No tag';
        }, $notAllowedTags));

        $allowedTags = implode(' ', array_map(static function ($tag) {
            return "<$tag>Tagged</$tag>";
        }, ElementParser::ALLOWED_TAGS));

        $xml = <<<XML
<element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">$notAllowedTagsXML $allowedTags</element>
XML;
        $this->shouldHaveContent($xml, "$notAllowedTagsParsed $allowedTags");
    }

    public function it_should_remove_forbidden_tags()
    {
        $this->setForbiddenTags(['ins']);

        $xml = <<<XML
<element>Here is a paragraph <span>with allowed tag.</span><ins id="dsg">This tag should be removed!!!</ins> Another paragraph.<ins id="dssdsdg">This should also be removed.</ins> End.</element>
XML;
        $this->shouldHaveContent(
            $xml,
            'Here is a paragraph <span>with allowed tag.</span> Another paragraph. End.'
        );
    }

    public function it_should_override_allowed_tags()
    {
        $this->setAllowedTags(['span', 'i', 'em', 'strong']);
        $content = implode(' ', [
            '<span>Tagged</span>',
            '<i>Tagged</i>',
            '<em>Tagged</em>',
            '<b>Tagged</b>',
            '<strong>Tagged</strong>',
            '<sup>Tagged</sup>',
            '<sub>Tagged</sub>',
            '<br>Tagged</br>',
            '<hr>Tagged</hr>',
            '<a>Tagged</a>',
            '<script>Tagged</script>',
            '<iframe>Tagged</iframe>'
        ]);

        $result = implode(' ', [
            '<span>Tagged</span>',
            '<i>Tagged</i>',
            '<em>Tagged</em>',
            'Tagged',
            '<strong>Tagged</strong>',
            'Tagged',
            'Tagged',
            'Tagged',
            'Tagged',
            'Tagged',
            'Tagged',
            'Tagged'
        ]);

        $xml = "<element>$content</element>";
        $this->shouldHaveContent($xml, $result);
    }

    private function shouldHaveContent($xml, $parsedContent): void
    {
        $this->validateXML($xml, ['content' => $parsedContent]);
    }

    protected function validateXML($xml, array $matchValues): void
    {
        $el = simplexml_load_string($xml, 'SimpleXMLIterator');
        $this->parse($el)->shouldReturnValidItem($matchValues);
    }
}
