<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use Infomaker\Everyware\Support\Format;
use Infomaker\Everyware\NewsML\Parsers\ImageLinkParser;
use Spec\Infomaker\Everyware\NewsML\ParserSpec;

class ImageLinkParserSpec extends ParserSpec
{
    function it_is_initializable()
    {
        $this->shouldHaveType(ImageLinkParser::class);
    }

    public function it_should_parse_a_link()
    {
        $xml = <<<XML
<link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
    <data>
        <text>Little fluffball Molly is at her happiest when she has her chew toy.</text>
        <width>300</width>
        <height>169</height>
        <imageInstructions>This image has rights on how to use it...</imageInstructions>
        <credit>Nyhetsbyrån</credit>
        <alttext>Image description</alttext>
        <alignment>left</alignment>
        <flags>
            <flag>disableAutomaticCrop</flag>
        </flags>
    </data>
    <links>
        <link rel="crop" type="x-im/crop" title="16:9" uri="im://crop/0/0/1/1"/>
        <link rel="crop" type="x-im/crop" title="3:2" uri="im://crop/0.07875/0/0.84375/1"/>
        <link rel="crop" type="x-im/crop" title="2:3" uri="im://crop/0.3125/0/0.375/1"/>
        <link rel="author" type="x-im/author" title="Ronnie J. WIllis" uuid="7a39b42b-1315-4711-a136-7b3a9f132110"/>
        <link rel="author" type="x-im/author" title="Jane Doe" uuid="00000000-0000-0000-0000-000000000000">
            <data>
                <email>jane.doe@email.com</email>
            </data>
        </link>
        <link rel="focal-point" type="x-im/focal-point">
            <data>
                <x>123</x>
                <y>456</y>
            </data>
        </link>
    </links>
</link>
XML;
        $this->validateXML($xml, [
            'rel' => 'self',
            'type' => 'x-im/image',
            'uri' => 'im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg',
            'uuid' => 'd83dad15-9476-54c1-a57d-495757664dae',
            'width' => '300',
            'height' => '169',
            'instructions' => 'This image has rights on how to use it...',
            'text' => 'Little fluffball Molly is at her happiest when she has her chew toy.',
            'alttext' => 'Image description',
            'credit' => 'Nyhetsbyrån',
            'alignment' => 'left',
            'crop' => [
                '16:9' => [
                    'x' => Format::formatNumberToImengineFloat(0),
                    'y' => Format::formatNumberToImengineFloat(0),
                    'width' => Format::formatNumberToImengineFloat(1),
                    'height' => Format::formatNumberToImengineFloat(1),
                ],
                '3:2' => [
                    'x' => Format::formatNumberToImengineFloat(0.07875),
                    'y' => Format::formatNumberToImengineFloat(0),
                    'width' => Format::formatNumberToImengineFloat(0.84375),
                    'height' => Format::formatNumberToImengineFloat(1),
                ],
                '2:3' => [
                    'x' => Format::formatNumberToImengineFloat(0.3125),
                    'y' => Format::formatNumberToImengineFloat(0),
                    'width' => Format::formatNumberToImengineFloat(0.375),
                    'height' => Format::formatNumberToImengineFloat(1),
                ],
            ],
            'authors' => [
                [
                    'title' => 'Ronnie J. WIllis',
                    'uuid' => '7a39b42b-1315-4711-a136-7b3a9f132110'
                ],
                [
                    'title' => 'Jane Doe',
                    'uuid' => '00000000-0000-0000-0000-000000000000',
                    'email' => 'jane.doe@email.com'
                ]
            ],
            'flags' => [
                ['name' => 'disableAutomaticCrop']
            ],
            'focal-point' => [
                ['x' => '123', 'y' => '456']
            ]
        ]);
    }

    public function it_should_have_default_values()
    {
        $xml = '<link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae"/>';
        $this->validateXML($xml, ImageLinkParser::DEFAULT_DATA);
    }

    public function it_should_have_text_markup()
    {
        $text = '<mark>hey</mark> <em>there</em><strong> general</strong> <a href="google.com">kenobi</a>!';

        $xml = <<<XML
<link rel="self" type="x-im/image" uri="im://image/5hdHkiVOHRmvLsNLTDYrUSUVS6E.png" uuid="d027b036-0b84-5287-b37a-901f6f8f54b4">
    <data>
        <text>$text</text>
    </data>
</link>
XML;
        $this->validateXML($xml, [
            "text" => $text
        ]);
    }

    public function it_should_have_authors()
    {
        $xml = <<<XML
<link rel="self" type="x-im/image" uri="im://image/5hdHkiVOHRmvLsNLTDYrUSUVS6E.png" uuid="d027b036-0b84-5287-b37a-901f6f8f54b4">
    <data>
    </data>
    <links>
        <link rel="author" uuid="5884e555-468b-41cd-a14b-d1768eb8ccf0" title="John Doe" type="x-im/author">
            <data>
                <email>john.doe@info.com</email>
            </data>
        </link>
        <link rel="author" uuid="2ca22e98-c418-4330-ae00-4fa2127cae2d" title="Jane Doe" type="x-im/author">
            <data>
                <email>jane.doe@navigaglobal.com</email>
            </data>
        </link>
    </links>
</link>
XML;
        $this->validateXML($xml, [
            'authors' => [
                [
                    'rel' => 'author',
                    'uuid' => '5884e555-468b-41cd-a14b-d1768eb8ccf0',
                    'title' => 'John Doe',
                    'type' => 'x-im/author',
                    'email' => 'john.doe@info.com'
                ],
                [
                    'rel' => 'author',
                    'uuid' => '2ca22e98-c418-4330-ae00-4fa2127cae2d',
                    'title' => 'Jane Doe',
                    'type' => 'x-im/author',
                    'email' => 'jane.doe@navigaglobal.com'
                ]
            ]
        ]);
    }

    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/image');
    }
}
