<?php declare(strict_types=1);

namespace Spec\Matchers;

use PhpSpec\Exception\Example\FailureException;
use PhpSpec\Matcher\BasicMatcher;

/**
 * Class BaseItemValidation
 * @package Spec\Matchers
 */
abstract class BaseItemValidation extends BasicMatcher
{

    protected function getFailureException(string $name, $subject, array $arguments): FailureException
    {
        return new FailureException(
            implode(PHP_EOL, [
                'expected',
                json_encode($arguments[0] ?? 'empty'),
                'but got ',
                json_encode($subject),
            ])
        );
    }

    protected function getNegativeFailureException(string $name, $subject, array $arguments): FailureException
    {
        return $this->getFailureException($name, $subject, $arguments);
    }

    public function supports(string $name, $subject, array $arguments): bool
    {
        return in_array($name, $this->supportedNames(), true) && count($arguments) === 1 && is_array($arguments[0]);
    }

    /**
     * @param string       $name
     * @param string|array $value
     * @param array        $properties
     *
     * @throws FailureException
     */
    private function validateProperty(string $name, $value, array $properties): void
    {
        if ( ! array_key_exists($name, $properties)) {
            throw new FailureException(
                sprintf('The Item did not match the desired property: "%s"', $name)
            );
        }

        $propertyMatch = $properties[$name];

        if (is_array($value)) {

            if ( ! is_array($propertyMatch)) {
                throw new FailureException(
                    sprintf('Expected property: "%s" to be an array, got %s', $name, gettype($propertyMatch))
                );
            }

            if (empty($value) && ! empty($propertyMatch)) {
                throw new FailureException(implode(PHP_EOL, [
                    sprintf('The property: "%s" does not match the desired value.', $name),
                    'expected ',
                    json_encode($value, JSON_PRETTY_PRINT),
                    'but got ',
                    json_encode($propertyMatch, JSON_PRETTY_PRINT)
                ]));
            }

            if ($this->isPropertyList($value)) {
                $this->matchProperties($value, $propertyMatch);
            } else {
                foreach ($value as $i => $expectedProperties) {
                    $this->matchProperties($expectedProperties, $propertyMatch[$i]);
                }
            }
        } elseif ($value !== $propertyMatch) {
            throw new FailureException(implode(PHP_EOL, [
                sprintf('The property: "%s" does not match the desired value.', $name),
                'expected ',
                "\"{$value}\"",
                'but got ',
                "\"{$propertyMatch}\""
            ]));
        }
    }

    /**
     * @param array $expectedProperties
     * @param array $parsedProperties
     *
     * @throws FailureException
     */
    protected function matchProperties(array $expectedProperties, array $parsedProperties): void
    {
        foreach ($expectedProperties as $name => $value) {
            $this->validateProperty($name, $value, $parsedProperties);
        }
    }

    protected function isPropertyList(array $propertyValue): bool
    {
        if (empty($propertyValue)) {
            return false;
        }

        foreach (array_keys($propertyValue) as $key) {
            if (is_string($key)) {
                return true;
            }
        }

        return false;
    }

    abstract protected function supportedNames(): array;
}
